package com.gildedrose;

import static org.assertj.core.api.Assertions.assertThat;

import java.util.Arrays;
import org.junit.jupiter.api.Test;

class GildedRoseTest {

  @Test
  void shouldUpdateForOneDay() {
    GildedRose rose = new GildedRose(ItemsProvider.get());

    rose.updateQuality();

    assertThat(rose.items).extracting(item -> item.sellIn).containsExactly(9, 1, 4, 0, -1, 14, 9, 4);
    assertThat(rose.items).extracting(item -> item.quality).containsExactly(19, 1, 6, 80, 80, 21, 50, 50);
  }

  @Test
  void shouldUpdateForTwoDays() {
    GildedRose rose = new GildedRose(ItemsProvider.get());

    rose.updateQuality();
    rose.updateQuality();

    assertThat(rose.items).extracting(item -> item.sellIn).containsExactly(8, 0, 3, 0, -1, 13, 8, 3);
    assertThat(rose.items).extracting(item -> item.quality).containsExactly(18, 2, 5, 80, 80, 22, 50, 50);
  }

  @Test
  void shouldBehaveLikeGoldenMaster() {
    GildedRose rose = new GildedRose(ItemsProvider.get());
    GildedRoseGolden golden = new GildedRoseGolden(ItemsProvider.get());

    for (int day = 0; day < 100; day++) {
      rose.updateQuality();
      golden.updateQuality();

      assertThat(rose.items)
        .extracting(item -> item.sellIn)
        .containsExactly(Arrays.stream(golden.items).map(item -> item.sellIn).toArray(Integer[]::new));
      assertThat(rose.items)
        .extracting(item -> item.quality)
        .containsExactly(Arrays.stream(golden.items).map(item -> item.quality).toArray(Integer[]::new));
    }
  }
}
